@@ -10,6 +10,7 @@ pub enum ErrorKind {
     MissingField,
     OutOfRange,
     WrongLength,
+    EndOfFile,
 }
 
 /// An error for deser.
@@ -17,6 +18,7 @@ pub enum ErrorKind {
 pub struct Error {
     kind: ErrorKind,
     msg: Cow<'static, str>,
+    source: Option<Box<dyn std::error::Error + Send + Sync>>,
 }
 
 impl Error {
@@ -25,9 +27,16 @@ impl Error {
         Error {
             kind,
             msg: msg.into(),
+            source: None,
         }
     }
 
+    /// Attaches another error as source to this error.
+    pub fn with_source<E: std::error::Error + Send + Sync + 'static>(mut self, source: E) -> Self {
+        self.source = Some(Box::new(source));
+        self
+    }
+
     /// Returns the kind of the error.
     pub fn kind(&self) -> ErrorKind {
         self.kind
@@ -40,4 +49,8 @@ impl fmt::Display for Error {
     }
 }
 
-impl std::error::Error for Error {}
+impl std::error::Error for Error {
+    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
+        self.source.as_ref().map(|err| err.as_ref() as _)
+    }
+}
