@@ -1,10 +1,10 @@
-use deser::de::Driver;
+use deser::de::DeserializeDriver;
 use deser::{Deserialize, Event};
 
 fn deserialize<T: Deserialize>(events: Vec<Event<'_>>) -> T {
     let mut out = None;
     {
-        let mut driver = Driver::new(&mut out);
+        let mut driver = DeserializeDriver::new(&mut out);
         for event in events {
             driver.emit(event).unwrap();
         }
@@ -443,3 +443,13 @@ fn test_flatten_incomplete_inner() {
         Event::MapEnd,
     ]);
 }
+
+#[test]
+fn test_deserializing_newtype() {
+    #[derive(Deserialize)]
+    struct MyInt(u32);
+
+    let x: MyInt = deserialize(vec![1u64.into()]);
+
+    assert_eq!(x.0, 1);
+}
